@@ -19,6 +19,7 @@
 #include <linux/interrupt.h>
 #include <linux/netdevice.h>
 #include <linux/skbuff.h>
+#include <linux/net_tstamp.h>
 #ifdef ENA_XDP_SUPPORT
 #include <net/xdp.h>
 #endif
@@ -26,12 +27,18 @@
 #include <uapi/linux/bpf.h>
 #endif
 #include <linux/u64_stats_sync.h>
+#ifdef ENA_HAS_DEVLINK_HEADERS
+#include <net/devlink.h>
+#endif /* ENA_HAS_DEVLINK_HEADERS */
+#ifdef ENA_PAGE_POOL_SUPPORT
+#include <net/page_pool/helpers.h>
+#endif /* ENA_PAGE_POOL_SUPPORT */
 
 #include "ena_com.h"
 #include "ena_eth_com.h"
 
 #define DRV_MODULE_GEN_MAJOR	2
-#define DRV_MODULE_GEN_MINOR	13
+#define DRV_MODULE_GEN_MINOR	16
 #define DRV_MODULE_GEN_SUBMINOR	0
 
 #define DRV_MODULE_NAME		"ena"
@@ -83,9 +90,6 @@
 
 #define ENA_PKT_MAX_BUFS	19
 
-#define ENA_RX_RSS_TABLE_LOG_SIZE  7
-#define ENA_RX_RSS_TABLE_SIZE	(1 << ENA_RX_RSS_TABLE_LOG_SIZE)
-
 /* The number of tx packet completions that will be handled each NAPI poll
  * cycle is ring_size / ENA_TX_POLL_BUDGET_DIVIDER.
  */
@@ -127,8 +131,10 @@
 
 #define ENA_MMIO_DISABLE_REG_READ	BIT(0)
 
+#ifdef ENA_LPC_SUPPORT
 struct ena_page_cache;
 
+#endif /* ENA_LPC_SUPPORT */
 #ifdef ENA_PHC_SUPPORT
 struct ena_phc_info;
 
@@ -145,6 +151,7 @@ struct ena_irq {
 struct ena_napi {
 	unsigned long last_intr_jiffies ____cacheline_aligned;
 	u8 interrupts_masked;
+	bool lost_interrupt_unmask_handled;
 	struct napi_struct napi;
 	struct ena_ring *tx_ring;
 	struct ena_ring *rx_ring;
@@ -175,8 +182,8 @@ struct ena_tx_buffer {
 	/* Indicate if bufs[0] map the linear data of the skb. */
 	u8 map_linear_data;
 
-	/* Used for detect missing tx packets to limit the number of prints */
-	u8 print_once;
+	/* Used for detecting missed tx packets */
+	bool timed_out;
 #ifdef ENA_AF_XDP_SUPPORT
 
 	/* used for ordering TX completions when needed (e.g. AF_XDP) */
@@ -214,7 +221,15 @@ struct ena_rx_buffer {
 	u32 page_offset;
 	u32 buf_offset;
 	struct ena_com_buf ena_buf;
+#ifdef ENA_LPC_SUPPORT
 	bool is_lpc_page;
+#endif /* ENA_LPC_SUPPORT */
+#ifdef ENA_PAGE_POOL_SUPPORT
+	/* Used to locally frag a page allocated from page pool via the DRB
+	 * mechanism, thus avoiding atomic ops to update the page ref.
+	 */
+	long pagecnt_bias;
+#endif /* ENA_PAGE_POOL_SUPPORT */
 } ____cacheline_aligned;
 
 struct ena_stats_tx {
@@ -232,14 +247,20 @@ struct ena_stats_tx {
 	u64 bad_req_id;
 	u64 llq_buffer_copy;
 	u64 missed_tx;
+	atomic64_t pending_timedout_pkts;
 	u64 unmask_interrupt;
 	u64 last_napi_jiffies;
+	u64 lost_interrupt;
 #ifdef ENA_AF_XDP_SUPPORT
 	u64 xsk_cnt;
 	u64 xsk_bytes;
 	u64 xsk_need_wakeup_set;
 	u64 xsk_wakeup_request;
 #endif /* ENA_AF_XDP_SUPPORT */
+#ifdef ENA_XDP_MB_SUPPORT
+	u64 xdp_frags_exceeded;
+	u64 xdp_short_linear_part;
+#endif /* ENA_XDP_MB_SUPPORT */
 };
 
 struct ena_stats_rx {
@@ -269,9 +290,26 @@ struct ena_stats_rx {
 	u64 xdp_invalid;
 	u64 xdp_redirect;
 #endif
+#ifdef ENA_LPC_SUPPORT
 	u64 lpc_warm_up;
 	u64 lpc_full;
 	u64 lpc_wrong_numa;
+#endif /* ENA_LPC_SUPPORT */
+#ifdef ENA_PAGE_POOL_SUPPORT
+#ifdef CONFIG_PAGE_POOL_STATS
+	u64 pp_alloc_fast;
+	u64 pp_alloc_slow;
+	u64 pp_alloc_slow_hi_ord;
+	u64 pp_alloc_empty;
+	u64 pp_alloc_refill;
+	u64 pp_alloc_waive;
+	u64 pp_cached;
+	u64 pp_cache_full;
+	u64 pp_ring;
+	u64 pp_ring_full;
+	u64 pp_released_ref;
+#endif /* CONFIG_PAGE_POOL_STATS */
+#endif /* ENA_PAGE_POOL_SUPPORT */
 #ifdef ENA_AF_XDP_SUPPORT
 	u64 xsk_need_wakeup_set;
 	u64 zc_queue_pkt_copy;
@@ -294,13 +332,25 @@ struct ena_ring {
 	struct pci_dev *pdev;
 	struct napi_struct *napi;
 	struct net_device *netdev;
+#ifdef ENA_LPC_SUPPORT
 	struct ena_page_cache *page_cache;
+#endif /* ENA_LPC_SUPPORT */
+#ifdef ENA_PAGE_POOL_SUPPORT
+	struct page_pool *page_pool;
+#endif /* ENA_PAGE_POOL_SUPPORT */
 	struct ena_com_dev *ena_dev;
 	struct ena_adapter *adapter;
 	struct ena_com_io_cq *ena_com_io_cq;
 	struct ena_com_io_sq *ena_com_io_sq;
 #ifdef ENA_XDP_SUPPORT
 	struct bpf_prog *xdp_bpf_prog;
+#ifdef ENA_XDP_MB_SUPPORT
+	bool xdp_prog_support_frags;
+#endif /* ENA_XDP_MB_SUPPORT */
+#endif /* ENA_XDP_SUPPORT */
+	unsigned long last_checked_last_napi_jiffies;
+	bool last_checked_is_cq_empty;
+#ifdef ENA_XDP_SUPPORT
 	struct xdp_rxq_info xdp_rxq;
 	spinlock_t xdp_tx_lock;	/* synchronize XDP TX/Redirect traffic */
 	/* Used for rx queues only to point to the xdp tx ring, to
@@ -365,7 +415,14 @@ enum ena_busy_poll_state_t {
 	ENA_BP_STATE_POLL,
 	ENA_BP_STATE_DISABLE
 };
+
 #endif
+struct ena_keep_alive_stats {
+	u64 rx_drops;
+	u64 tx_drops;
+	u64 rx_overruns;
+};
+
 struct ena_stats_dev {
 	u64 tx_timeout;
 	u64 suspend;
@@ -374,9 +431,6 @@ struct ena_stats_dev {
 	u64 interface_up;
 	u64 interface_down;
 	u64 admin_q_pause;
-	u64 rx_drops;
-	u64 tx_drops;
-	u64 rx_overruns;
 	u64 reset_fail;
 	u64 total_resets;
 	u64 bad_tx_req_id;
@@ -393,6 +447,7 @@ struct ena_stats_dev {
 	u64 admin_to;
 	u64 device_request_reset;
 	u64 missing_first_intr;
+	struct ena_keep_alive_stats ka_stats;
 };
 
 enum ena_flags_t {
@@ -415,6 +470,12 @@ enum ena_llq_header_size_policy_t {
 	ENA_LLQ_HEADER_SIZE_POLICY_LARGE
 };
 
+struct hw_timestamp_state {
+	struct hwtstamp_config ts_cfg;
+	u8 hw_tx_supported;
+	u8 hw_rx_supported;
+};
+
 /* adapter specific private data structure */
 struct ena_adapter {
 	struct ena_com_dev *ena_dev;
@@ -431,11 +492,13 @@ struct ena_adapter {
 	u32 num_io_queues;
 	u32 max_num_io_queues;
 
+#ifdef ENA_LPC_SUPPORT
 	/* Local page cache size when it's enabled */
 	u32 configured_lpc_size;
 	/* Current Local page cache size */
 	u32 used_lpc_size;
 
+#endif /* ENA_LPC_SUPPORT */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 8, 0)
 	struct msix_entry *msix_entries;
 #endif
@@ -497,8 +560,6 @@ struct ena_adapter {
 
 	struct u64_stats_sync syncp;
 	struct ena_stats_dev dev_stats;
-	struct ena_admin_eni_stats eni_stats;
-	struct ena_admin_ena_srd_info ena_srd_info;
 
 	/* last queue index that was checked for missing completions / interrupts */
 	u32 last_monitored_qid;
@@ -510,6 +571,18 @@ struct ena_adapter {
 #endif
 	u32 xdp_first_ring;
 	u32 xdp_num_queues;
+
+	struct hw_timestamp_state hw_ts_state;
+	struct ena_keep_alive_stats persistent_ka_stats;
+
+	struct devlink *devlink;
+#ifdef ENA_DEVLINK_SUPPORT
+	struct devlink_port devlink_port;
+#endif /* ENA_DEVLINK_SUPPORT */
+#ifdef CONFIG_DEBUG_FS
+
+	struct dentry *debugfs_base;
+#endif /* CONFIG_DEBUG_FS */
 };
 
 #define ENA_RESET_STATS_ENTRY(reset_reason, stat) \
@@ -547,8 +620,12 @@ void ena_dump_stats_to_dmesg(struct ena_adapter *adapter);
 
 void ena_dump_stats_to_buf(struct ena_adapter *adapter, u8 *buf);
 
+struct sk_buff *ena_alloc_skb(struct ena_ring *rx_ring, void *first_frag, u16 len);
+
+#ifdef ENA_LPC_SUPPORT
 int ena_set_lpc_state(struct ena_adapter *adapter, bool enabled);
 
+#endif /* ENA_LPC_SUPPORT */
 int ena_update_queue_params(struct ena_adapter *adapter,
 			    u32 new_tx_size,
 			    u32 new_rx_size,
@@ -657,6 +734,44 @@ static inline void ena_reset_device(struct ena_adapter *adapter,
 	set_bit(ENA_FLAG_TRIGGER_RESET, &adapter->flags);
 }
 
+static inline int ena_tx_map_frags(struct skb_shared_info *sh_info,
+				   struct ena_tx_buffer *tx_info,
+				   struct ena_ring *tx_ring,
+				   struct ena_com_buf *ena_buf,
+				   u16 delta)
+{
+	u32 nr_frags = sh_info->nr_frags;
+	dma_addr_t dma;
+	u32 frag_len;
+	int i, rc;
+
+	for (i = 0; i < nr_frags; i++) {
+		const skb_frag_t *frag = &sh_info->frags[i];
+
+		frag_len = skb_frag_size(frag);
+
+		if (unlikely(delta >= frag_len)) {
+			delta -= frag_len;
+			continue;
+		}
+
+		dma = skb_frag_dma_map(tx_ring->dev, frag, delta,
+				       frag_len - delta, DMA_TO_DEVICE);
+		rc = dma_mapping_error(tx_ring->dev, dma);
+		if (unlikely(rc))
+			return rc;
+
+		ena_buf->paddr = dma;
+		ena_buf->len = frag_len - delta;
+		ena_buf++;
+		tx_info->num_of_bufs++;
+		delta = 0;
+	}
+
+	return 0;
+}
+
+#ifdef ENA_LPC_SUPPORT
 /* Allocate a page and DMA map it
  * @rx_ring: The IO queue pair which requests the allocation
  *
@@ -665,6 +780,7 @@ static inline void ena_reset_device(struct ena_adapter *adapter,
  */
 struct page *ena_alloc_map_page(struct ena_ring *rx_ring, dma_addr_t *dma);
 
+#endif /* ENA_LPC_SUPPORT */
 int ena_destroy_device(struct ena_adapter *adapter, bool graceful);
 int ena_restore_device(struct ena_adapter *adapter);
 void ena_get_and_dump_head_tx_cdesc(struct ena_com_io_cq *io_cq);
@@ -675,7 +791,7 @@ int validate_tx_req_id(struct ena_ring *tx_ring, u16 req_id);
 
 static inline void ena_ring_tx_doorbell(struct ena_ring *tx_ring)
 {
-	ena_com_write_sq_doorbell(tx_ring->ena_com_io_sq);
+	ena_com_write_tx_sq_doorbell(tx_ring->ena_com_io_sq);
 	ena_increase_stat(&tx_ring->tx_stats.doorbells, 1, &tx_ring->syncp);
 }
 
@@ -691,8 +807,25 @@ void ena_init_io_rings(struct ena_adapter *adapter,
 		       int first_index, int count);
 void ena_down(struct ena_adapter *adapter);
 int ena_up(struct ena_adapter *adapter);
-void ena_unmask_interrupt(struct ena_ring *tx_ring, struct ena_ring *rx_ring);
+void ena_unmask_interrupt(struct ena_ring *tx_ring,
+			  struct ena_ring *rx_ring,
+			  bool lost_interrupt);
 void ena_update_ring_numa_node(struct ena_ring *rx_ring);
+void ena_unmap_rx_buff_attrs(struct ena_ring *rx_ring,
+			     struct ena_rx_buffer *rx_info,
+			     unsigned long attrs);
+struct sk_buff *ena_rx_skb_copybreak(struct ena_ring *rx_ring,
+				     struct ena_rx_buffer *rx_info,
+				     u16 len, u8 meta_len, int pkt_offset,
+				     void *buf_data_addr);
+void ena_fill_rx_frags(struct ena_ring *rx_ring,
+		       u32 descs,
+		       struct ena_com_rx_buf_info *ena_bufs,
+		       struct skb_shared_info *sh_info,
+#ifdef ENA_XDP_MB_SUPPORT
+		       struct xdp_buff *xdp,
+#endif /* ENA_XDP_MB_SUPPORT */
+		       struct sk_buff *skb);
 void ena_rx_checksum(struct ena_ring *rx_ring,
 		     struct ena_com_rx_ctx *ena_rx_ctx,
 		     struct sk_buff *skb);
@@ -712,4 +845,65 @@ static inline void handle_tx_comp_poll_error(struct ena_ring *tx_ring, u16 req_i
 	}
 }
 
+static inline u32 get_rss_indirection_table_size(struct ena_adapter *adapter)
+{
+	struct ena_com_dev *ena_dev = adapter->ena_dev;
+
+	if (!ena_com_indirection_table_config_supported(ena_dev))
+		return 0;
+
+	return (1UL << ena_dev->rss.tbl_log_size);
+}
+
+static inline void ena_rx_release_packet_buffers(struct ena_ring *rx_ring,
+						 int first_to_release,
+						 int last_to_release)
+{
+	int i;
+
+	for (i = first_to_release; i <= last_to_release; i++) {
+		int req_id = rx_ring->ena_bufs[i].req_id;
+
+#ifndef ENA_PAGE_POOL_SUPPORT
+		/* The page pool mechanism unmaps page when the page is freed
+		 * from the pool.
+		 */
+		ena_unmap_rx_buff_attrs(rx_ring,
+					&rx_ring->rx_buffer_info[req_id],
+					ENA_DMA_ATTR_SKIP_CPU_SYNC);
+#endif /* ENA_PAGE_POOL_SUPPORT */
+		rx_ring->rx_buffer_info[req_id].page = NULL;
+	}
+}
+
+static inline bool ena_too_many_tx_frags(u8 nr_frags, u16 sgl_size,
+					 u32 header_len, u8 tx_max_header_size,
+					 bool is_llq)
+{
+	if (likely(nr_frags < sgl_size))
+		return false;
+
+	/* If the number of frags is the maximum allowed number of bufs in a
+	 * tx packet then usually the buffer holding the linear part of the
+	 * skb/xdp_frame will add another buffer to the packet, increasing the
+	 * number of buffers in the packet over the allowed limit.
+	 *
+	 * Except for the following cases:
+	 * 1. In LLQ case: If the linear part of the skb/xdp_frame fits the
+	 *    header part of the LLQ entry perfectly, the linear part of the
+	 *    skb/xdp_frame will not be used as the first buffer.
+	 * 2. In non-LLQ case: if the size of the linear part is 0, then the
+	 *    linear part will not become an extra buffer.
+	 */
+	if (unlikely(nr_frags == sgl_size)) {
+		if (likely(is_llq && (header_len <= tx_max_header_size)))
+			return false;
+
+		if (unlikely(!is_llq && !header_len))
+			return false;
+	}
+
+	return true;
+}
+
 #endif /* !(ENA_H) */
