@@ -47,13 +47,7 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #endif
 
 #include <asm/io.h>
-
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33) )
-#include <linux/utsrelease.h>
-#else
 #include <generated/utsrelease.h>
-#endif
-
 #include <linux/delay.h>
 #include <linux/errno.h>
 #include <linux/etherdevice.h>
@@ -80,9 +74,7 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #include <linux/bitfield.h>
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,6,0)
 #include <linux/sizes.h>
-#endif
 
 /* For ACCESS_ONCE, WRITE_ONCE and READ_ONCE macros */
 #include<linux/compiler.h>
@@ -212,38 +204,12 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #ifndef RHEL_RELEASE_VERSION
 #define RHEL_RELEASE_VERSION(a,b) (((a) << 8) + (b))
 #endif
-#ifndef AX_RELEASE_VERSION
-#define AX_RELEASE_VERSION(a,b) (((a) << 8) + (b))
-#endif
-
-#ifndef AX_RELEASE_CODE
-#define AX_RELEASE_CODE 0
-#endif
 
 #ifndef RHEL_RELEASE_CODE
 #define RHEL_RELEASE_CODE 0
 #endif
 
-#if (AX_RELEASE_CODE && AX_RELEASE_CODE == AX_RELEASE_VERSION(3,0))
-#define RHEL_RELEASE_CODE RHEL_RELEASE_VERSION(5,0)
-#elif (AX_RELEASE_CODE && AX_RELEASE_CODE == AX_RELEASE_VERSION(3,1))
-#define RHEL_RELEASE_CODE RHEL_RELEASE_VERSION(5,1)
-#elif (AX_RELEASE_CODE && AX_RELEASE_CODE == AX_RELEASE_VERSION(3,2))
-#define RHEL_RELEASE_CODE RHEL_RELEASE_VERSION(5,3)
-#endif
-
 /*****************************************************************************/
-#if (RHEL_RELEASE_CODE && \
-	(RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,6)) && \
-     (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(7,0)))
-#define HAVE_RHEL6_NET_DEVICE_OPS_EXT
-#endif
-
-#if (RHEL_RELEASE_CODE && \
-	(RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,4)) && \
-     (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(7,0)))
-#define HAVE_RHEL6_ETHTOOL_OPS_EXT_STRUCT
-#endif /* RHEL >= 6.4 && RHEL < 7.0 */
 
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(4,10,0) || \
 	(SLE_VERSION_CODE && \
@@ -259,86 +225,10 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #define NDO_GET_STATS_64_V2
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0) || \
-	(RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,5))
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
 #include <net/busy_poll.h>
 #endif
 
-/*****************************************************************************/
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,37) )
-/* The function netif_set_real_num_tx_queues() doesn't return value for
- * kernels < 2.6.37
- */
-static inline int _kc_netif_set_real_num_tx_queues(struct net_device *dev,
-                                                   unsigned int txq)
-{
-        netif_set_real_num_tx_queues(dev, txq);
-        return 0;
-}
-#define netif_set_real_num_tx_queues(dev, txq) \
-        _kc_netif_set_real_num_tx_queues(dev, txq)
-
-#endif /* < 2.6.37 */
-
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(3,3,0) )
-#if !(RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,5))
-typedef u32 netdev_features_t;
-#endif
-#undef PCI_EXP_TYPE_RC_EC
-#define  PCI_EXP_TYPE_RC_EC	0xa	/* Root Complex Event Collector */
-#ifndef CONFIG_BQL
-#define netdev_tx_completed_queue(_q, _p, _b) do {} while (0)
-#define netdev_completed_queue(_n, _p, _b) do {} while (0)
-#define netdev_tx_sent_queue(_q, _b) do {} while (0)
-#define netdev_sent_queue(_n, _b) do {} while (0)
-#define netdev_tx_reset_queue(_q) do {} while (0)
-#define netdev_reset_queue(_n) do {} while (0)
-#endif
-
-#endif /* < 3.3.0 */
-
-/******************************************************************************/
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(3,4,0) )
-#ifdef NET_ADDR_RANDOM
-#define eth_hw_addr_random(N) do { \
-	eth_random_addr(N->dev_addr); \
-	N->addr_assign_type |= NET_ADDR_RANDOM; \
-	} while (0)
-#else /* NET_ADDR_RANDOM */
-#define eth_hw_addr_random(N) eth_random_addr(N->dev_addr)
-#endif /* NET_ADDR_RANDOM */
-#if !(RHEL_RELEASE_CODE)
-/* If probe retry doesn't define, return no device */
-#define EPROBE_DEFER ENODEV
-#endif
-#endif /* >= 3.4.0 */
-
-/*****************************************************************************/
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(3,5,0) )
-#if !(RHEL_RELEASE_CODE)
-static inline bool ether_addr_equal(const u8 *addr1, const u8 *addr2)
-{
-	const u16 *a = (const u16 *)addr1;
-	const u16 *b = (const u16 *)addr2;
-
-	return ((a[0] ^ b[0]) | (a[1] ^ b[1]) | (a[2] ^ b[2])) == 0;
-}
-#endif
-#endif /* >= 3.5.0 */
-
-/******************************************************************************/
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0) )
-#ifndef eth_random_addr
-#define eth_random_addr _kc_eth_random_addr
-static inline void _kc_eth_random_addr(u8 *addr)
-{
-        get_random_bytes(addr, ETH_ALEN);
-        addr[0] &= 0xfe; /* clear multicast */
-        addr[0] |= 0x02; /* set local assignment */
-}
-#endif
-#endif /* < 3.6.0 */
-
 /******************************************************************************/
 #ifndef CONFIG_NET_RX_BUSY_POLL
 static inline void skb_mark_napi_id(struct sk_buff *skb,
@@ -358,21 +248,6 @@ static inline void napi_hash_add(struct napi_struct *napi)
 }
 #endif /* CONFIG_NET_RX_BUSY_POLL */
 
-/*****************************************************************************/
-#if ( LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) )
-/* cpu_rmap is buggy on older version and causes dead lock */
-#ifdef CONFIG_RFS_ACCEL
-#undef CONFIG_RFS_ACCEL
-#endif
-
-#if !(RHEL_RELEASE_CODE)
-static inline u32 ethtool_rxfh_indir_default(u32 index, u32 n_rx_rings)
-{
-	return index % n_rx_rings;
-}
-#endif
-#endif /* >= 3.8.0 */
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,13,0)
 #if BITS_PER_LONG == 32 && defined(CONFIG_SMP)
 # define u64_stats_init(syncp)  seqcount_init(syncp.seq)
@@ -381,10 +256,8 @@ static inline u32 ethtool_rxfh_indir_default(u32 index, u32 n_rx_rings)
 #endif
 
 #if !(SLE_VERSION_CODE && SLE_VERSION_CODE >= SLE_VERSION(12,0,0)) && \
-	!(RHEL_RELEASE_CODE && ((RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,8) && \
-	                        (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(7,0))) \
-                            || (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,1)))) && \
-     !defined(UEK3_RELEASE)
+    !(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7,0))) && \
+    !defined(UEK3_RELEASE)
 static inline void reinit_completion(struct completion *x)
 {
          x->done = 0;
@@ -394,8 +267,7 @@ static inline void reinit_completion(struct completion *x)
 #endif /* < 3.13.0 */
 
 #if  (( LINUX_VERSION_CODE < KERNEL_VERSION(3,13,0) ) && \
-     (!(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6,0) && \
-       RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7,0))) \
+     (!(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7,0)) \
      && !(SLE_VERSION_CODE && SLE_VERSION_CODE >= SLE_VERSION(12,0,0))&& \
      !defined(UEK3_RELEASE))) || \
      (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE < UBUNTU_VERSION(3,13,0,30))
@@ -425,15 +297,6 @@ static inline int pci_enable_msix_range(struct pci_dev *dev,
 }
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3,13,0) && \
-    !(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE > RHEL_RELEASE_VERSION(7,1))
-static inline void *devm_kcalloc(struct device *dev,
-				 size_t n, size_t size, gfp_t flags)
-{
-	return devm_kzalloc(dev, n * size, flags | __GFP_ZERO);
-}
-#endif
-
 /*****************************************************************************/
 #if (( LINUX_VERSION_CODE < KERNEL_VERSION(3,13,8) ) && \
      !RHEL_RELEASE_CODE && \
@@ -456,8 +319,7 @@ static inline void skb_set_hash(struct sk_buff *skb, __u32 hash,
 
 /*****************************************************************************/
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0)
-#if !(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7,0) && \
-			        RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(6,6)) \
+#if !(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7,0)) \
     && !(defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE >= UBUNTU_VERSION(3,13,0,105))
 static inline int pci_msix_vec_count(struct pci_dev *dev)
 {
@@ -481,10 +343,6 @@ static inline void ether_addr_copy(u8 *dst, const u8 *src)
 #endif /* RHEL 7 */
 #endif
 
-#if (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(6,8)))
-#define napi_gro_flush(napi, flush_old) napi_gro_flush(napi)
-#endif
-
 #if ( LINUX_VERSION_CODE >= KERNEL_VERSION(3,15,0) || \
 	(defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE >= UBUNTU_VERSION(3,13,0,30))) || \
 	(SLE_VERSION_CODE && SLE_VERSION_CODE >= SLE_VERSION(12,0,0)) || \
@@ -581,9 +439,7 @@ static inline void __napi_schedule_irqoff(struct napi_struct *n)
 /*****************************************************************************/
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,0,0) \
-	|| (RHEL_RELEASE_CODE && ((RHEL_RELEASE_CODE > RHEL_RELEASE_VERSION(6,7)) && \
-	                          (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(7,0))) \
-	                      || RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,2)) \
+	|| (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,2))) \
 	|| (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE >= UBUNTU_VERSION(3,19,0,51))
 #else
 static inline void napi_complete_done(struct napi_struct *n, int work_done)
@@ -724,10 +580,9 @@ do {									\
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,19,0) && \
-    !(RHEL_RELEASE_CODE && ((RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7, 1)) && \
-			    (RHEL_RELEASE_CODE > RHEL_RELEASE_VERSION(6, 6)))) && \
-			    !defined(UBUNTU_VERSION_CODE) && \
-			    !defined(UEK3_RELEASE) && (!defined(DEBIAN_VERSION) || DEBIAN_VERSION != 8)
+    !(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7, 1))) && \
+    !defined(UBUNTU_VERSION_CODE) && \
+    !defined(UEK3_RELEASE) && (!defined(DEBIAN_VERSION) || DEBIAN_VERSION != 8)
 
 #define DO_ONCE(func, ...)						     \
 	({								     \
@@ -763,6 +618,24 @@ static inline void netdev_rss_key_fill(void *buffer, size_t len)
 #define READ_ONCE(x) ACCESS_ONCE(x)
 #endif
 
+#ifndef ENA_HAVE_ATOMIC_SET_RELEASE
+#ifdef ENA_HAVE_SMP_STORE_RELEASE
+#define atomic_set_release(p, v) do {		\
+	smp_store_release(&(p)->counter, (v));	\
+} while (0)
+#else /* ENA_HAVE_SMP_STORE_RELEASE */
+#define atomic_set_release(p, v) do {					\
+	/* Using smp_mb(), ensures all read/write operations are	\
+	 * completed before proceeding to write value v to address p.	\
+	 * The most strict implementation is chosen to ensure release	\
+	 * semantics on all platforms and kernel configurations.	\
+	 */								\
+	smp_mb();							\
+	WRITE_ONCE((p)->counter, v);					\
+} while (0)
+#endif /* ENA_HAVE_SMP_STORE_RELEASE */
+#endif /* ENA_HAVE_ATOMIC_SET_RELEASE */
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9 ,0)
 #define ENA_GENERIC_PM_OPS
 #endif
@@ -814,25 +687,6 @@ static inline struct page *dev_alloc_page(void)
 }
 #endif
 
-/* This entry might seem strange because of the #ifndef numa_mem_id(),
- * but these defines were taken from the Linux kernel
- */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 34)
-#ifndef numa_mem_id
-#ifdef CONFIG_HAVE_MEMORYLESS_NODES
-static inline int numa_mem_id(void)
-{
-	return __this_cpu_read(_numa_mem_);
-}
-#else /* CONFIG_HAVE_MEMORYLESS_NODES */
-static inline int numa_mem_id(void)
-{
-	return numa_node_id();
-}
-#endif /* CONFIG_HAVE_MEMORYLESS_NODES */
-#endif /* numa_mem_id */
-#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 34) */
-
 #ifndef fallthrough
 #define fallthrough do {} while (0)  /* fallthrough */
 #endif
@@ -842,10 +696,6 @@ static inline int numa_mem_id(void)
 #define AF_XDP_BUSY_POLL_SUPPORTED
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
-#define ENA_LINEAR_FRAG_SUPPORTED
-#endif
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
 #define ENA_NETDEV_LOGS_WITHOUT_RV
 #endif
@@ -917,16 +767,6 @@ static inline void eth_hw_addr_set(struct net_device *dev, const u8 *addr)
 #define ENA_AF_XDP_SUPPORT
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 3, 0)
-/* kernels older than 3.3.0 didn't have this function and
- * used netif_tx_queue_stopped() for the same purpose
- */
-static inline int netif_xmit_stopped(const struct netdev_queue *dev_queue)
-{
-	return netif_tx_queue_stopped(dev_queue);
-}
-#endif
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0)
 #define NAPIF_STATE_SCHED BIT(NAPI_STATE_SCHED)
 #endif
@@ -944,21 +784,8 @@ static inline int netif_xmit_stopped(const struct netdev_queue *dev_queue)
 #define HAS_BPF_HEADER
 #endif
 
-#if ((LINUX_VERSION_CODE < KERNEL_VERSION(3, 8, 0)) && \
-	!(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 7))))
-static inline int ktime_compare(const ktime_t cmp1, const ktime_t cmp2)
-{
-	if (cmp1.tv64 < cmp2.tv64)
-		return -1;
-	if (cmp1.tv64 > cmp2.tv64)
-		return 1;
-	return 0;
-}
-#endif
-
 #if ((LINUX_VERSION_CODE < KERNEL_VERSION(3, 16, 0)) && \
 	!(RHEL_RELEASE_CODE && \
-	(RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 7)) && \
 	(RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7, 0)) && \
 	(RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7, 1))))
 static inline bool ktime_after(const ktime_t cmp1, const ktime_t cmp2)
@@ -969,8 +796,7 @@ static inline bool ktime_after(const ktime_t cmp1, const ktime_t cmp2)
 
 #if IS_ENABLED(CONFIG_PTP_1588_CLOCK)
 
-#if defined(ENA_PHC_INCLUDE) && ((LINUX_VERSION_CODE >= KERNEL_VERSION(3, 0, 0)) || \
-	(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 4)))
+#if defined(ENA_PHC_INCLUDE) && (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 0, 0))
 #define ENA_PHC_SUPPORT
 #endif /* ENA_PHC_SUPPORT */
 
@@ -986,15 +812,6 @@ static inline bool ktime_after(const ktime_t cmp1, const ktime_t cmp2)
 #define ENA_PHC_SUPPORT_GETTIME64_EXTENDED
 #endif /* ENA_PHC_SUPPORT_GETTIME64_EXTENDED */
 
-#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(3, 0, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0)))
-#define ENA_PHC_SUPPORT_ADJFREQ
-#endif /* ENA_PHC_SUPPORT_ADJFREQ */
-
-#if ((LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)) && \
-	!(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 4))))
-#define ptp_clock_register(info, parent) ptp_clock_register(info)
-#endif
-
 #endif /* CONFIG_PTP_1588_CLOCK */
 
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(3, 19, 0)) && \
@@ -1019,8 +836,12 @@ static inline ssize_t strscpy(char *dest, const char *src, size_t count)
 
 static inline void ena_netif_napi_add(struct net_device *dev,
 				      struct napi_struct *napi,
-				      int (*poll)(struct napi_struct *, int))
+				      int (*poll)(struct napi_struct *, int),
+				      int index)
 {
+#ifdef ENA_NETIF_NAPI_ADD_CONFIG
+	netif_napi_add_config(dev, napi, poll, index);
+#else
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0)) && \
 	!(RHEL_RELEASE_CODE && \
 	  ((RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 8)) && \
@@ -1033,6 +854,7 @@ static inline void ena_netif_napi_add(struct net_device *dev,
 #else
 	netif_napi_add(dev, napi, poll);
 #endif /* LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0) */
+#endif /* ENA_NETIF_NAPI_ADD_CONFIG */
 }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 4, 0)
@@ -1046,15 +868,6 @@ static inline void ena_netif_napi_add(struct net_device *dev,
 #define FIELD_PREP(mask, value) ((typeof(mask))(((value) << (__builtin_ffsll(mask) - 1)) & (mask)))
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0)
-#define xdp_features_set_redirect_target(netdev, xdp_xmit_supported)
-#define xdp_features_clear_redirect_target(netdev)
-#define xdp_clear_features_flag(netdev)
-#define xdp_set_features_flag(netdev, features)
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0) */
-#define ENA_XDP_NETLINK_ADVERTISEMENT
-#endif
-
 #if (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(7, 4))) || \
     (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE < UBUNTU_VERSION(4, 5, 0, 0)) || \
     (defined(IS_UEK) && !ENA_KERNEL_VERSION_GTE(4, 1, 12, 105, 0, 0))
@@ -1078,8 +891,6 @@ static inline void dma_unmap_page_attrs(struct device *dev,
     (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE < UBUNTU_VERSION(4, 5, 0, 0)) || \
     (defined(IS_UEK) && LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 13))
 #define ENA_DMA_ATTR_SKIP_CPU_SYNC (1 << DMA_ATTR_SKIP_CPU_SYNC)
-#elif (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(6, 10)))
-#define	ENA_DMA_ATTR_SKIP_CPU_SYNC 0
 #else
 #define ENA_DMA_ATTR_SKIP_CPU_SYNC DMA_ATTR_SKIP_CPU_SYNC
 #endif
@@ -1166,4 +977,117 @@ static inline int irq_update_affinity_hint(unsigned int irq, const struct cpumas
 #define RX_CLS_FLOW_WAKE	0xfffffffffffffffeULL
 #endif /* RX_CLS_FLOW_WAKE */
 
+#ifndef ENA_HAVE_XDP_FEATURES_SET_REDIRECT_TARGET
+#define xdp_features_set_redirect_target(netdev, xdp_xmit_supported)
+#define xdp_features_clear_redirect_target(netdev)
+#endif /* ENA_HAVE_XDP_FEATURES_SET_REDIRECT_TARGET */
+
+#ifndef ENA_HAVE_XDP_SET_FEATURES_FLAG
+#define xdp_clear_features_flag(netdev)
+#define xdp_set_features_flag(netdev, features)
+#endif /* ENA_HAVE_XDP_SET_FEATURES_FLAG */
+
+#if defined(ENA_XDP_SUPPORT) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
+#ifdef ENA_AF_XDP_SUPPORT
+#include <net/xdp_sock_drv.h>
+#endif /* ENA_AF_XDP_SUPPORT */
+static inline int ena_xdp_return_buff(struct xdp_buff *xdp)
+{
+	struct xdp_frame *xdpf;
+
+#ifdef ENA_AF_XDP_SUPPORT
+	if (xdp->rxq->mem.type == MEM_TYPE_XSK_BUFF_POOL) {
+		xsk_buff_free(xdp);
+		return 0;
+	}
+#endif
+	/* Non zero-copy pages should be safe to convert to
+	 * an xdp_frame, which has xdp_return_frame() function exported
+	 * to all kernels that support XDP
+	 */
+#ifdef XDP_CONVERT_TO_FRAME_NAME_CHANGED
+	xdpf = xdp_convert_buff_to_frame(xdp);
+#else
+	xdpf = convert_to_xdp_frame(xdp);
+#endif /* XDP_CONVERT_TO_FRAME_NAME_CHANGED */
+	if (!xdpf)
+		return -EOVERFLOW;
+
+	xdp_return_frame(xdpf);
+
+	return 0;
+}
+#elif defined(ENA_XDP_SUPPORT)
+static inline int ena_xdp_return_buff(struct xdp_buff *xdp)
+{
+	/* If xdp_return_buff() is defined it cannot fail */
+	xdp_return_buff(xdp);
+
+	return 0;
+}
+#endif /* defined(ENA_XDP_SUPPORT) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 17, 0) */
+
+#if defined(ENA_XDP_SUPPORT) && defined(ENA_HAVE_XDP_MB_DEPS)
+#define ENA_XDP_MB_SUPPORT
+#endif
+
+#if defined(ENA_XDP_MB_SUPPORT) && !defined(ENA_HAVE_SKB_FRAG_FILL_PAGE_DESC)
+static inline void skb_frag_fill_page_desc(skb_frag_t *frag,
+					   struct page *page,
+					   int off, int size)
+{
+	frag->bv_page = page;
+	frag->bv_offset = off;
+	skb_frag_size_set(frag, size);
+}
+#endif /* defined(ENA_XDP_MB_SUPPORT) && !defined(ENA_HAVE_SKB_FRAG_FILL_PAGE_DESC) */
+
+#ifdef ENA_XDP_SUPPORT
+static inline int ena_xdp_rxq_info_reg(struct xdp_rxq_info *xdp_rxq,
+				       struct net_device *dev, u32 queue_index,
+				       unsigned int napi_id, u32 frag_size)
+{
+#ifdef ENA_XDP_MB_SUPPORT
+	return __xdp_rxq_info_reg(xdp_rxq, dev, queue_index, napi_id, frag_size);
+
+#else /* ENA_XDP_MB_SUPPORT */
+
+#ifdef AF_XDP_BUSY_POLL_SUPPORTED
+#ifdef ENA_AF_XDP_SUPPORT
+	return xdp_rxq_info_reg(xdp_rxq, dev, queue_index, napi_id);
+#else
+	return xdp_rxq_info_reg(xdp_rxq, dev, queue_index, 0);
+#endif /* ENA_AF_XDP_SUPPORT */
+#else
+	return xdp_rxq_info_reg(xdp_rxq, dev, queue_index);
+#endif /* AF_XDP_BUSY_POLL_SUPPORTED */
+
+#endif /* ENA_XDP_MB_SUPPORT */
+}
+#endif /* ENA_XDP_SUPPORT */
+
+#ifndef ENA_HAVE_SKB_METADATA_SET
+static inline void skb_metadata_set(struct sk_buff *skb, u8 meta_len) {}
+#endif /* ENA_HAVE_SKB_METADATA_SET */
+
+#ifndef timer_container_of
+#define timer_container_of(adapter, t, timer_service) from_timer(adapter, t, timer_service)
+#endif /* timer_container_of */
+
+#if defined(ENA_DEVLINK_INCLUDE) && defined(CONFIG_NET_DEVLINK) && LINUX_VERSION_CODE >= KERNEL_VERSION(6, 12, 0)
+#define ENA_DEVLINK_SUPPORT
+#endif /* ENA_DEVLINK_INCLUDE && CONFIG_NET_DEVLINK && KERNEL >= 6.12 */
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 12, 0)
+#define ENA_PAGE_POOL_SUPPORT
+#else
+#define ENA_LPC_SUPPORT
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(6, 12, 0) */
+
+#ifdef ENA_PAGE_POOL_SUPPORT
+#define ENA_XDP_MEM_TYPE MEM_TYPE_PAGE_POOL
+#else
+#define ENA_XDP_MEM_TYPE MEM_TYPE_PAGE_SHARED
+#endif /* ENA_PAGE_POOL_SUPPORT */
+
 #endif /* _KCOMPAT_H_ */
